const navbar = document.getElementById("navbar")
const navigator = document.getElementById("navigator")
const views = {
    draw: {
        index: "wgt-package/views/draw.html",
        controller: DrawViewController,
    },
    camera: {
        index: "wgt-package/views/camera.html",
        controller: CameraViewController,
    },
    train: {
        index: "wgt-package/views/train.html",
        controller: TrainViewController,
    },
    logs: {
        index: "wgt-package/views/logs.html",
        controller: LogsViewController,
    },
}
let currentController = null


async function loadView(name) {
    if(currentController && "dispose" in currentController) {
        currentController.dispose()
    }

    if(!(name in views)) {
        navigator.innerHTML = `Unknown view: ${name}`
        return
    }

    const view = views[name]

    if(view.cache) {
        navigator.innerHTML = view.cache
    }
    else {
        try {
            const html = await new Promise((resolve, reject) => {
                try {
                    const file = tizen.filesystem.openFile(view.index, "r")
                    file.readStringNonBlocking(html => {
                        file.close()
                        resolve(html)
                    }, error => {
                        file.close()
                        reject(error)
                    })
                }
                catch(error) {
                    reject(error)
                }
            })
            navigator.innerHTML = html
            view.cache = html
        }
        catch(error) {
            Logger.instance.error(`Error loading view: ${error}`)
            navigator.innerHTML = `Error loading view: ${error}`
            return
        }
    }

    currentController = new view.controller(navigator)
}


window.onload = async () => {
    document.addEventListener('tizenhwkey', (e) => {
        if (e.keyName === 'back') {
            try {
                tizen.application.getCurrentApplication().exit()
            } catch (ignore) {}
        }
    })

    const result = await requestPermissionsAll([
        "http://tizen.org/privilege/mediastorage",
        "http://tizen.org/privilege/filesystem.read",
        "http://tizen.org/privilege/filesystem.write",
    ])

    Logger.instance.debug(JSON.stringify(result, null, 2))

    if(result.all === true) {
        MnistModel.instance.open({
            modelPath: "wgt-private/trained_model.ini",
            trainDatasetPath: "documents/mnist/train.dat",
            testDatasetPath: "documents/mnist/test.dat",
        }).then(() => {
            Logger.instance.info("Loaded model")
        }).catch(error => {
            Logger.instance.error(`Error loading model: ${error}`)
        })
    }
    else {
        Logger.instance.info("Storage access denied")
    }

    const anchors = navbar.querySelectorAll("a")
    anchors.forEach(a => {
        const viewName = a.href.split("#")[1]
        if(a.classList.contains("active")) {
            loadView(viewName)
        }
        a.addEventListener("click", () => {
            anchors.forEach(a => a.classList.remove("active"))
            a.classList.add("active")
            loadView(viewName)
        })
    })
}
